/**
 * *##% jaxx-runtime-swing-widget
 * Copyright (C) 2008 - 2009 CodeLutin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 *
 * You should have received a copy of the GNU General Lesser Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>. ##%*
 */
package jaxx.runtime.swing.editor.config.model;

import org.nuiton.util.ApplicationConfig.OptionDef;

/**
 * le modele d'une option de la configuration a editer.
 *
 * @author tchemit
 */
public class OptionModel implements OptionDef {

    /** la definition de l'option ( venant de la config) */
    protected final OptionDef def;
    /** un drapeau pour savoir si l'option est valide (n'est pas utilisé actuellement) */
    protected boolean valid = true;
    /** un drapeau pour savoir si l'option a été sauvée */
    protected boolean saved = false;
    /** la valeur non modifié de l'option */
    protected Object originalValue;
    /** la valeur actuelle de l'option (peut être la valeur orignal si non modifée)*/
    protected Object value;

    protected OptionModel(OptionDef def, Object value) {
        this.def = def;
        initValue(value);
    }

    @Override
    public String getKey() {
        return def.getKey();
    }

    @Override
    public Class<?> getType() {
        return def.getType();
    }

    @Override
    public String getDescription() {
        return def.getDescription();
    }

    @Override
    public String getDefaultValue() {
        return def.getDefaultValue();
    }

    @Override
    public boolean isTransient() {
        return def.isTransient();
    }

    @Override
    public boolean isFinal() {
        return def.isFinal();
    }

    public Object getOriginalValue() {
        return originalValue;
    }

    public Object getValue() {
        return value;
    }

    public void setValue(Object value) {
        this.value = value;
    }

    public boolean isModified() {
        return !originalValue.equals(value);
    }

    public boolean isValid() {
        return valid;
    }

    public void setValid(boolean valid) {
        this.valid = valid;
    }

    public boolean isSaved() {
        return saved;
    }

    public void setSaved(boolean saved) {
        this.saved = saved;
    }
    
    public void initValue(Object originalValue) {
        this.originalValue = originalValue;
        this.value = originalValue;
    }
}
